==> playship-fruitninja-1.0.0.tgz: package/src/lifecycle.js <==
const logger = require("./common").createLogger();
const stringify = require("json-stringify");
const randomitem = require("random-item");
const SPAWN_DETAILS_KEY = "spwanDetails";
const END_REASON = "Game Had Ended";
const spawnNewFruits = function(game) {
  let spawnDetails = game.getTransient(SPAWN_DETAILS_KEY);
  if (!spawnDetails) return logger.error("spawn details object not found!!");
  if (spawnDetails.nextSpawnTime > Date.now()) return;
  logger.trace("spawning time....");
  let spawnCount = 5;
  if (this.meta && this.meta.extras.spawnCount) {
    spawnCount = this.meta.extras.spawnCount;
  }
  logger.trace("Will spawn %s fruits ", spawnCount);
  //generate a random array of spawn fruit indexes

  let fruitsToBeSpawned = [];
  for (let c = 0; c < spawnCount; c++) {
    fruitsToBeSpawned.push(randomitem(this.fruitTypes));
  }
  if (logger.trace()) {
    logger.trace("will spawn fruits %s ", stringify(fruitsToBeSpawned));
  }
  sendSpawnOrder.call(this, game, fruitsToBeSpawned);
  spawnDetails.nextSpawnTime = Date.now() + this.meta.extras.spawnFrequency;
};

const endGame = function(game) {
  let endTime = game.getScheduledStartTime() + game.getDuration();
  if (endTime < Date.now()) {
    //end game
    logger.debug("Ending fruit ninja instance %s", game.getId());
    game.endGame(END_REASON, err => {
      if (err) {
        logger.error({ err: err });
        logger.error("Error in ending game");
      }
    });
    return true;
  }
  return false;
};
const step = function(game) {
  let gameState = game.getState();
  switch (gameState) {
    case 20: //running
      if (endGame.call(this, game)) return;
      spawnNewFruits.call(this, game);
      break;
    default:
      logger.trace("gamestate is %s, nothing to do ", gameState);
  }
};

const sendSpawnOrder = function(game, fruitsToBeSpawned) {
  let ts = Date.now();
  let connectedPlayers = game.getConnectedPlayers();
  let expiryTime = Date.now() + this.meta.extras.spawnFrequency * 10;
  connectedPlayers.forEach(player => {
    let gameObjects = [];
    fruitsToBeSpawned.forEach(type => {
      let gameObject = {
        type: type,
        id: player + "_" + type + "_" + ts,
        owner: player,
        expiryTime: expiryTime
      };
      gameObjects.push(gameObject);
    });
    game.spawnObjects(gameObjects, false, player);
  });
};

const initFruitTypes = function(meta) {
  if (!meta) return;
  //create an object type array
  let fruitTypes = [];
  meta.gameObjects.forEach(obj => {
    if (obj.type === "blade") return;
    fruitTypes.push(obj.type);
  });
  this.fruitTypes = fruitTypes;
  this.meta = meta;
};

const startGame = function(game, cb, meta) {
  let spawnDetails = { nextSpawnTime: Date.now() };
  game.addTransient(SPAWN_DETAILS_KEY, spawnDetails);
  cb(null, null);
};

const resumeGame = function(game) {
  logger.info("resuming game ");
  game.addTransient(SPAWN_DETAILS_KEY, { nextSpawnTime: Date.now() });
};

const gameLoaded = function(meta) {
  logger.info("loading defs from meta %s", stringify(meta));
  initFruitTypes.call(this, meta);
};

const gameResult = function(game) {
  //get all playerIds
  if (game.getState() !== 30) {
    return [];
  }
  let playerIds = game.getAllPlayerIds();
  if (logger.debug()) {
    logger.debug("playerd who played %s", stringify(playerIds));
  }
  if (playerIds == null || playerIds == undefined || playerIds.length !== 2) {
    logger.error("Fatal error, player count != 2 ");
    throw { err: "INVALID_PLAYER_COUNT" };
  }
  let p1Score = game.getPlayerScore(playerIds[0]);
  let p2Score = game.getPlayerScore(playerIds[1]);
  logger.info(
    "scores for players %s=%s, %s=%s",
    playerIds[0],
    p1Score,
    playerIds[1],
    p2Score
  );
  if (p1Score === p2Score) {
    return [
      { playerId: playerIds[0], winType: 3 },
      { playerId: playerIds[1], winType: 3 }
    ];
  }
  if (p1Score < p2Score) {
    return [
      { playerId: playerIds[0], winType: 2 },
      { playerId: playerIds[1], winType: 1 }
    ];
  }
  return [
    { playerId: playerIds[0], winType: 1 },
    { playerId: playerIds[1], winType: 2 }
  ];
};

module.exports = { startGame, step, gameLoaded, resumeGame, gameResult };

==> playship-fruitninja-1.0.1.tgz: package/src/lifecycle.js <==
const logger = require("./common").createLogger();
const stringify = require("json-stringify");
const randomitem = require("random-item");
const SPAWN_DETAILS_KEY = "spwanDetails";
const END_REASON = "Game Had Ended";
const spawnNewFruits = function(game) {
  let spawnDetails = game.getTransient(SPAWN_DETAILS_KEY);
  if (!spawnDetails) return logger.error("spawn details object not found!!");
  if (spawnDetails.nextSpawnTime > Date.now()) return;
  logger.trace("spawning time....");
  let spawnCount = 5;
  if (this.meta && this.meta.extras.spawnCount) {
    spawnCount = this.meta.extras.spawnCount;
  }
  logger.trace("Will spawn %s fruits ", spawnCount);
  //generate a random array of spawn fruit indexes

  let fruitsToBeSpawned = [];
  for (let c = 0; c < spawnCount; c++) {
    fruitsToBeSpawned.push(randomitem(this.fruitTypes));
  }
  if (logger.trace()) {
    logger.trace("will spawn fruits %s ", stringify(fruitsToBeSpawned));
  }
  sendSpawnOrder.call(this, game, fruitsToBeSpawned);
  spawnDetails.nextSpawnTime = Date.now() + this.meta.extras.spawnFrequency;
};

const endGame = function(game) {
  let endTime = game.getScheduledStartTime() + game.getDuration();
  if (endTime < Date.now()) {
    //end game
    logger.debug("Ending fruit ninja instance %s", game.getId());
    game.endGame(END_REASON, err => {
      if (err) {
        logger.error({ err: err });
        logger.error("Error in ending game");
      }
    });
    return true;
  }
  return false;
};
const step = function(game) {
  let gameState = game.getState();
  switch (gameState) {
    case 20: //running
      if (endGame.call(this, game)) return;
      spawnNewFruits.call(this, game);
      break;
    default:
      logger.trace("gamestate is %s, nothing to do ", gameState);
  }
};

const sendSpawnOrder = function(game, fruitsToBeSpawned) {
  let ts = Date.now();
  let connectedPlayers = game.getConnectedPlayers();
  let expiryTime = Date.now() + this.meta.extras.spawnFrequency * 10;
  connectedPlayers.forEach(player => {
    let gameObjects = [];
    for (let i = 0; i < fruitsToBeSpawned.length; i++) {
      let type = fruitsToBeSpawned[i];
      let gameObject = {
        type: type,
        id: player + "_" + type + "_" + (ts + i),
        owner: player,
        expiryTime: expiryTime
      };
      gameObjects.push(gameObject);
    }
    game.spawnObjects(gameObjects, false, player);
  });
};

const initFruitTypes = function(meta) {
  if (!meta) return;
  //create an object type array
  let fruitTypes = [];
  meta.gameObjects.forEach(obj => {
    if (obj.type === "blade") return;
    fruitTypes.push(obj.type);
  });
  this.fruitTypes = fruitTypes;
  this.meta = meta;
};

const startGame = function(game, cb, meta) {
  let spawnDetails = { nextSpawnTime: Date.now() };
  game.addTransient(SPAWN_DETAILS_KEY, spawnDetails);
  cb(null, null);
};

const resumeGame = function(game) {
  logger.info("resuming game ");
  game.addTransient(SPAWN_DETAILS_KEY, { nextSpawnTime: Date.now() });
};

const gameLoaded = function(meta) {
  logger.info("loading defs from meta %s", stringify(meta));
  initFruitTypes.call(this, meta);
};

const gameResult = function(game) {
  //get all playerIds
  if (game.getState() !== 30) {
    return [];
  }
  let playerIds = game.getAllPlayerIds();
  if (logger.debug()) {
    logger.debug("playerd who played %s", stringify(playerIds));
  }
  if (playerIds == null || playerIds == undefined || playerIds.length !== 2) {
    logger.error("Fatal error, player count != 2 ");
    throw { err: "INVALID_PLAYER_COUNT" };
  }
  let p1Score = game.getPlayerScore(playerIds[0]);
  let p2Score = game.getPlayerScore(playerIds[1]);
  logger.info(
    "scores for players %s=%s, %s=%s",
    playerIds[0],
    p1Score,
    playerIds[1],
    p2Score
  );
  if (p1Score === p2Score) {
    return [
      { playerId: playerIds[0], winType: 3 },
      { playerId: playerIds[1], winType: 3 }
    ];
  }
  if (p1Score < p2Score) {
    return [
      { playerId: playerIds[0], winType: 2 },
      { playerId: playerIds[1], winType: 1 }
    ];
  }
  return [
    { playerId: playerIds[0], winType: 1 },
    { playerId: playerIds[1], winType: 2 }
  ];
};

module.exports = { startGame, step, gameLoaded, resumeGame, gameResult };
